# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...types.intel import ip_get_params
from ..._base_client import make_request_options
from ...types.intel.ip_get_response import IPGetResponse

__all__ = ["IPsResource", "AsyncIPsResource"]


class IPsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> IPsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return IPsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> IPsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return IPsResourceWithStreamingResponse(self)

    def get(
        self,
        *,
        account_id: str,
        ipv4: str | Omit = omit,
        ipv6: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[IPGetResponse]:
        """
        Gets the geolocation, ASN, infrastructure type of the ASN, and any security
        threat categories of an IP address. **Must provide ip query parameters.** For
        example, `/intel/ip?ipv4=1.1.1.1` or `/intel/ip?ipv6=2001:db8::1`.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/intel/ip", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ipv4": ipv4,
                        "ipv6": ipv6,
                    },
                    ip_get_params.IPGetParams,
                ),
                post_parser=ResultWrapper[Optional[IPGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[IPGetResponse]], ResultWrapper[IPGetResponse]),
        )


class AsyncIPsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncIPsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncIPsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncIPsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncIPsResourceWithStreamingResponse(self)

    async def get(
        self,
        *,
        account_id: str,
        ipv4: str | Omit = omit,
        ipv6: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[IPGetResponse]:
        """
        Gets the geolocation, ASN, infrastructure type of the ASN, and any security
        threat categories of an IP address. **Must provide ip query parameters.** For
        example, `/intel/ip?ipv4=1.1.1.1` or `/intel/ip?ipv6=2001:db8::1`.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/intel/ip", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "ipv4": ipv4,
                        "ipv6": ipv6,
                    },
                    ip_get_params.IPGetParams,
                ),
                post_parser=ResultWrapper[Optional[IPGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[IPGetResponse]], ResultWrapper[IPGetResponse]),
        )


class IPsResourceWithRawResponse:
    def __init__(self, ips: IPsResource) -> None:
        self._ips = ips

        self.get = to_raw_response_wrapper(
            ips.get,
        )


class AsyncIPsResourceWithRawResponse:
    def __init__(self, ips: AsyncIPsResource) -> None:
        self._ips = ips

        self.get = async_to_raw_response_wrapper(
            ips.get,
        )


class IPsResourceWithStreamingResponse:
    def __init__(self, ips: IPsResource) -> None:
        self._ips = ips

        self.get = to_streamed_response_wrapper(
            ips.get,
        )


class AsyncIPsResourceWithStreamingResponse:
    def __init__(self, ips: AsyncIPsResource) -> None:
        self._ips = ips

        self.get = async_to_streamed_response_wrapper(
            ips.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["MessagePushResponse", "Metadata", "MetadataMetrics"]


class MetadataMetrics(BaseModel):
    """Best-effort metrics for the queue.

    Values may be approximate due to the distributed nature of queues.
    """

    backlog_bytes: float
    """The size in bytes of unacknowledged messages in the queue."""

    backlog_count: float
    """The number of unacknowledged messages in the queue."""

    oldest_message_timestamp_ms: float
    """Unix timestamp in milliseconds of the oldest unacknowledged message in the
    queue.

    Returns 0 if unknown.
    """


class Metadata(BaseModel):
    metrics: Optional[MetadataMetrics] = None
    """Best-effort metrics for the queue.

    Values may be approximate due to the distributed nature of queues.
    """


class MessagePushResponse(BaseModel):
    metadata: Optional[Metadata] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .events import (
    EventsResource,
    AsyncEventsResource,
    EventsResourceWithRawResponse,
    AsyncEventsResourceWithRawResponse,
    EventsResourceWithStreamingResponse,
    AsyncEventsResourceWithStreamingResponse,
)
from .analytics import (
    AnalyticsResource,
    AsyncAnalyticsResource,
    AnalyticsResourceWithRawResponse,
    AsyncAnalyticsResourceWithRawResponse,
    AnalyticsResourceWithStreamingResponse,
    AsyncAnalyticsResourceWithStreamingResponse,
)
from .aggregates import (
    AggregatesResource,
    AsyncAggregatesResource,
    AggregatesResourceWithRawResponse,
    AsyncAggregatesResourceWithRawResponse,
    AggregatesResourceWithStreamingResponse,
    AsyncAggregatesResourceWithStreamingResponse,
)

__all__ = [
    "AggregatesResource",
    "AsyncAggregatesResource",
    "AggregatesResourceWithRawResponse",
    "AsyncAggregatesResourceWithRawResponse",
    "AggregatesResourceWithStreamingResponse",
    "AsyncAggregatesResourceWithStreamingResponse",
    "EventsResource",
    "AsyncEventsResource",
    "EventsResourceWithRawResponse",
    "AsyncEventsResourceWithRawResponse",
    "EventsResourceWithStreamingResponse",
    "AsyncEventsResourceWithStreamingResponse",
    "AnalyticsResource",
    "AsyncAnalyticsResource",
    "AnalyticsResourceWithRawResponse",
    "AsyncAnalyticsResourceWithRawResponse",
    "AnalyticsResourceWithStreamingResponse",
    "AsyncAnalyticsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["IndexQueryResponse", "Match"]


class Match(BaseModel):
    id: Optional[str] = None
    """Identifier for a Vector"""

    metadata: Optional[object] = None

    namespace: Optional[str] = None

    score: Optional[float] = None
    """The score of the vector according to the index's distance metric"""

    values: Optional[List[float]] = None


class IndexQueryResponse(BaseModel):
    count: Optional[int] = None
    """Specifies the count of vectors returned by the search"""

    matches: Optional[List[Match]] = None
    """Array of vectors matched by the search"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ....._models import BaseModel

__all__ = ["ItemDeleteResponse"]


class ItemDeleteResponse(BaseModel):
    key: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .status import (
    StatusResource,
    AsyncStatusResource,
    StatusResourceWithRawResponse,
    AsyncStatusResourceWithRawResponse,
    StatusResourceWithStreamingResponse,
    AsyncStatusResourceWithStreamingResponse,
)
from .outgoing import (
    OutgoingResource,
    AsyncOutgoingResource,
    OutgoingResourceWithRawResponse,
    AsyncOutgoingResourceWithRawResponse,
    OutgoingResourceWithStreamingResponse,
    AsyncOutgoingResourceWithStreamingResponse,
)

__all__ = [
    "StatusResource",
    "AsyncStatusResource",
    "StatusResourceWithRawResponse",
    "AsyncStatusResourceWithRawResponse",
    "StatusResourceWithStreamingResponse",
    "AsyncStatusResourceWithStreamingResponse",
    "OutgoingResource",
    "AsyncOutgoingResource",
    "OutgoingResourceWithRawResponse",
    "AsyncOutgoingResourceWithRawResponse",
    "OutgoingResourceWithStreamingResponse",
    "AsyncOutgoingResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from .outputs import (
    OutputsResource,
    AsyncOutputsResource,
    OutputsResourceWithRawResponse,
    AsyncOutputsResourceWithRawResponse,
    OutputsResourceWithStreamingResponse,
    AsyncOutputsResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.stream import live_input_list_params, live_input_create_params, live_input_update_params
from ....types.stream.live_input import LiveInput
from ....types.stream.live_input_list_response import LiveInputListResponse

__all__ = ["LiveInputsResource", "AsyncLiveInputsResource"]


class LiveInputsResource(SyncAPIResource):
    @cached_property
    def outputs(self) -> OutputsResource:
        return OutputsResource(self._client)

    @cached_property
    def with_raw_response(self) -> LiveInputsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return LiveInputsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LiveInputsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return LiveInputsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        default_creator: str | Omit = omit,
        delete_recording_after_days: float | Omit = omit,
        enabled: bool | Omit = omit,
        meta: object | Omit = omit,
        recording: live_input_create_params.Recording | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[LiveInput]:
        """
        Creates a live input, and returns credentials that you or your users can use to
        stream live video to Cloudflare Stream.

        Args:
          account_id: Identifier.

          default_creator: Sets the creator ID asssociated with this live input.

          delete_recording_after_days: Indicates the number of days after which the live inputs recordings will be
              deleted. When a stream completes and the recording is ready, the value is used
              to calculate a scheduled deletion date for that recording. Omit the field to
              indicate no change, or include with a `null` value to remove an existing
              scheduled deletion.

          enabled: Indicates whether the live input is enabled and can accept streams.

          meta: A user modifiable key-value store used to reference other systems of record for
              managing live inputs.

          recording: Records the input to a Cloudflare Stream video. Behavior depends on the mode. In
              most cases, the video will initially be viewable as a live video and transition
              to on-demand after a condition is satisfied.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/stream/live_inputs", account_id=account_id),
            body=maybe_transform(
                {
                    "default_creator": default_creator,
                    "delete_recording_after_days": delete_recording_after_days,
                    "enabled": enabled,
                    "meta": meta,
                    "recording": recording,
                },
                live_input_create_params.LiveInputCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[LiveInput]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[LiveInput]], ResultWrapper[LiveInput]),
        )

    def update(
        self,
        live_input_identifier: str,
        *,
        account_id: str,
        default_creator: str | Omit = omit,
        delete_recording_after_days: float | Omit = omit,
        enabled: bool | Omit = omit,
        meta: object | Omit = omit,
        recording: live_input_update_params.Recording | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[LiveInput]:
        """
        Updates a specified live input.

        Args:
          account_id: Identifier.

          live_input_identifier: A unique identifier for a live input.

          default_creator: Sets the creator ID asssociated with this live input.

          delete_recording_after_days: Indicates the number of days after which the live inputs recordings will be
              deleted. When a stream completes and the recording is ready, the value is used
              to calculate a scheduled deletion date for that recording. Omit the field to
              indicate no change, or include with a `null` value to remove an existing
              scheduled deletion.

          enabled: Indicates whether the live input is enabled and can accept streams.

          meta: A user modifiable key-value store used to reference other systems of record for
              managing live inputs.

          recording: Records the input to a Cloudflare Stream video. Behavior depends on the mode. In
              most cases, the video will initially be viewable as a live video and transition
              to on-demand after a condition is satisfied.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not live_input_identifier:
            raise ValueError(
                f"Expected a non-empty value for `live_input_identifier` but received {live_input_identifier!r}"
            )
        return self._put(
            path_template(
                "/accounts/{account_id}/stream/live_inputs/{live_input_identifier}",
                account_id=account_id,
                live_input_identifier=live_input_identifier,
            ),
            body=maybe_transform(
                {
                    "default_creator": default_creator,
                    "delete_recording_after_days": delete_recording_after_days,
                    "enabled": enabled,
                    "meta": meta,
                    "recording": recording,
                },
                live_input_update_params.LiveInputUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[LiveInput]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[LiveInput]], ResultWrapper[LiveInput]),
        )

    def list(
        self,
        *,
        account_id: str,
        include_counts: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[LiveInputListResponse]:
        """Lists the live inputs created for an account.

        To get the credentials needed to
        stream to a specific live input, request a single live input.

        Args:
          account_id: Identifier.

          include_counts: Includes the total number of videos associated with the submitted query
              parameters.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/stream/live_inputs", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"include_counts": include_counts}, live_input_list_params.LiveInputListParams),
                post_parser=ResultWrapper[Optional[LiveInputListResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[LiveInputListResponse]], ResultWrapper[LiveInputListResponse]),
        )

    def delete(
        self,
        live_input_identifier: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Prevents a live input from being streamed to and makes the live input
        inaccessible to any future API calls.

        Args:
          account_id: Identifier.

          live_input_identifier: A unique identifier for a live input.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not live_input_identifier:
            raise ValueError(
                f"Expected a non-empty value for `live_input_identifier` but received {live_input_identifier!r}"
            )
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            path_template(
                "/accounts/{account_id}/stream/live_inputs/{live_input_identifier}",
                account_id=account_id,
                live_input_identifier=live_input_identifier,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def get(
        self,
        live_input_identifier: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[LiveInput]:
        """
        Retrieves details of an existing live input.

        Args:
          account_id: Identifier.

          live_input_identifier: A unique identifier for a live input.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not live_input_identifier:
            raise ValueError(
                f"Expected a non-empty value for `live_input_identifier` but received {live_input_identifier!r}"
            )
        return self._get(
            path_template(
                "/accounts/{account_id}/stream/live_inputs/{live_input_identifier}",
                account_id=account_id,
                live_input_identifier=live_input_identifier,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[LiveInput]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[LiveInput]], ResultWrapper[LiveInput]),
        )


class AsyncLiveInputsResource(AsyncAPIResource):
    @cached_property
    def outputs(self) -> AsyncOutputsResource:
        return AsyncOutputsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncLiveInputsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncLiveInputsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLiveInputsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncLiveInputsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        default_creator: str | Omit = omit,
        delete_recording_after_days: float | Omit = omit,
        enabled: bool | Omit = omit,
        meta: object | Omit = omit,
        recording: live_input_create_params.Recording | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[LiveInput]:
        """
        Creates a live input, and returns credentials that you or your users can use to
        stream live video to Cloudflare Stream.

        Args:
          account_id: Identifier.

          default_creator: Sets the creator ID asssociated with this live input.

          delete_recording_after_days: Indicates the number of days after which the live inputs recordings will be
              deleted. When a stream completes and the recording is ready, the value is used
              to calculate a scheduled deletion date for that recording. Omit the field to
              indicate no change, or include with a `null` value to remove an existing
              scheduled deletion.

          enabled: Indicates whether the live input is enabled and can accept streams.

          meta: A user modifiable key-value store used to reference other systems of record for
              managing live inputs.

          recording: Records the input to a Cloudflare Stream video. Behavior depends on the mode. In
              most cases, the video will initially be viewable as a live video and transition
              to on-demand after a condition is satisfied.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/stream/live_inputs", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "default_creator": default_creator,
                    "delete_recording_after_days": delete_recording_after_days,
                    "enabled": enabled,
                    "meta": meta,
                    "recording": recording,
                },
                live_input_create_params.LiveInputCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[LiveInput]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[LiveInput]], ResultWrapper[LiveInput]),
        )

    async def update(
        self,
        live_input_identifier: str,
        *,
        account_id: str,
        default_creator: str | Omit = omit,
        delete_recording_after_days: float | Omit = omit,
        enabled: bool | Omit = omit,
        meta: object | Omit = omit,
        recording: live_input_update_params.Recording | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[LiveInput]:
        """
        Updates a specified live input.

        Args:
          account_id: Identifier.

          live_input_identifier: A unique identifier for a live input.

          default_creator: Sets the creator ID asssociated with this live input.

          delete_recording_after_days: Indicates the number of days after which the live inputs recordings will be
              deleted. When a stream completes and the recording is ready, the value is used
              to calculate a scheduled deletion date for that recording. Omit the field to
              indicate no change, or include with a `null` value to remove an existing
              scheduled deletion.

          enabled: Indicates whether the live input is enabled and can accept streams.

          meta: A user modifiable key-value store used to reference other systems of record for
              managing live inputs.

          recording: Records the input to a Cloudflare Stream video. Behavior depends on the mode. In
              most cases, the video will initially be viewable as a live video and transition
              to on-demand after a condition is satisfied.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not live_input_identifier:
            raise ValueError(
                f"Expected a non-empty value for `live_input_identifier` but received {live_input_identifier!r}"
            )
        return await self._put(
            path_template(
                "/accounts/{account_id}/stream/live_inputs/{live_input_identifier}",
                account_id=account_id,
                live_input_identifier=live_input_identifier,
            ),
            body=await async_maybe_transform(
                {
                    "default_creator": default_creator,
                    "delete_recording_after_days": delete_recording_after_days,
                    "enabled": enabled,
                    "meta": meta,
                    "recording": recording,
                },
                live_input_update_params.LiveInputUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[LiveInput]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[LiveInput]], ResultWrapper[LiveInput]),
        )

    async def list(
        self,
        *,
        account_id: str,
        include_counts: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[LiveInputListResponse]:
        """Lists the live inputs created for an account.

        To get the credentials needed to
        stream to a specific live input, request a single live input.

        Args:
          account_id: Identifier.

          include_counts: Includes the total number of videos associated with the submitted query
              parameters.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/stream/live_inputs", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"include_counts": include_counts}, live_input_list_params.LiveInputListParams
                ),
                post_parser=ResultWrapper[Optional[LiveInputListResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[LiveInputListResponse]], ResultWrapper[LiveInputListResponse]),
        )

    async def delete(
        self,
        live_input_identifier: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Prevents a live input from being streamed to and makes the live input
        inaccessible to any future API calls.

        Args:
          account_id: Identifier.

          live_input_identifier: A unique identifier for a live input.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not live_input_identifier:
            raise ValueError(
                f"Expected a non-empty value for `live_input_identifier` but received {live_input_identifier!r}"
            )
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            path_template(
                "/accounts/{account_id}/stream/live_inputs/{live_input_identifier}",
                account_id=account_id,
                live_input_identifier=live_input_identifier,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def get(
        self,
        live_input_identifier: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[LiveInput]:
        """
        Retrieves details of an existing live input.

        Args:
          account_id: Identifier.

          live_input_identifier: A unique identifier for a live input.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not live_input_identifier:
            raise ValueError(
                f"Expected a non-empty value for `live_input_identifier` but received {live_input_identifier!r}"
            )
        return await self._get(
            path_template(
                "/accounts/{account_id}/stream/live_inputs/{live_input_identifier}",
                account_id=account_id,
                live_input_identifier=live_input_identifier,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[LiveInput]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[LiveInput]], ResultWrapper[LiveInput]),
        )


class LiveInputsResourceWithRawResponse:
    def __init__(self, live_inputs: LiveInputsResource) -> None:
        self._live_inputs = live_inputs

        self.create = to_raw_response_wrapper(
            live_inputs.create,
        )
        self.update = to_raw_response_wrapper(
            live_inputs.update,
        )
        self.list = to_raw_response_wrapper(
            live_inputs.list,
        )
        self.delete = to_raw_response_wrapper(
            live_inputs.delete,
        )
        self.get = to_raw_response_wrapper(
            live_inputs.get,
        )

    @cached_property
    def outputs(self) -> OutputsResourceWithRawResponse:
        return OutputsResourceWithRawResponse(self._live_inputs.outputs)


class AsyncLiveInputsResourceWithRawResponse:
    def __init__(self, live_inputs: AsyncLiveInputsResource) -> None:
        self._live_inputs = live_inputs

        self.create = async_to_raw_response_wrapper(
            live_inputs.create,
        )
        self.update = async_to_raw_response_wrapper(
            live_inputs.update,
        )
        self.list = async_to_raw_response_wrapper(
            live_inputs.list,
        )
        self.delete = async_to_raw_response_wrapper(
            live_inputs.delete,
        )
        self.get = async_to_raw_response_wrapper(
            live_inputs.get,
        )

    @cached_property
    def outputs(self) -> AsyncOutputsResourceWithRawResponse:
        return AsyncOutputsResourceWithRawResponse(self._live_inputs.outputs)


class LiveInputsResourceWithStreamingResponse:
    def __init__(self, live_inputs: LiveInputsResource) -> None:
        self._live_inputs = live_inputs

        self.create = to_streamed_response_wrapper(
            live_inputs.create,
        )
        self.update = to_streamed_response_wrapper(
            live_inputs.update,
        )
        self.list = to_streamed_response_wrapper(
            live_inputs.list,
        )
        self.delete = to_streamed_response_wrapper(
            live_inputs.delete,
        )
        self.get = to_streamed_response_wrapper(
            live_inputs.get,
        )

    @cached_property
    def outputs(self) -> OutputsResourceWithStreamingResponse:
        return OutputsResourceWithStreamingResponse(self._live_inputs.outputs)


class AsyncLiveInputsResourceWithStreamingResponse:
    def __init__(self, live_inputs: AsyncLiveInputsResource) -> None:
        self._live_inputs = live_inputs

        self.create = async_to_streamed_response_wrapper(
            live_inputs.create,
        )
        self.update = async_to_streamed_response_wrapper(
            live_inputs.update,
        )
        self.list = async_to_streamed_response_wrapper(
            live_inputs.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            live_inputs.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            live_inputs.get,
        )

    @cached_property
    def outputs(self) -> AsyncOutputsResourceWithStreamingResponse:
        return AsyncOutputsResourceWithStreamingResponse(self._live_inputs.outputs)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .pattern import Pattern as Pattern
from .pattern_param import PatternParam as PatternParam
from .predefined_profile import PredefinedProfile as PredefinedProfile
from .custom_create_params import CustomCreateParams as CustomCreateParams
from .custom_update_params import CustomUpdateParams as CustomUpdateParams
from .predefined_update_params import PredefinedUpdateParams as PredefinedUpdateParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel

__all__ = ["TopupCreateResponse"]


class TopupCreateResponse(BaseModel):
    client_secret: Optional[str] = None
    """Stripe PaymentIntent client secret."""

    onboarding: bool
    """Whether the user was already onboarded."""

    payment_intent_id: str
    """Stripe invoice ID."""

    brand: Optional[str] = None
    """Card brand (visa, mastercard, etc.)."""

    last4: Optional[str] = None
    """Last 4 digits of card."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from .regions import (
    RegionsResource,
    AsyncRegionsResource,
    RegionsResourceWithRawResponse,
    AsyncRegionsResourceWithRawResponse,
    RegionsResourceWithStreamingResponse,
    AsyncRegionsResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncSinglePage, AsyncSinglePage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.addressing import regional_hostname_edit_params, regional_hostname_create_params
from ....types.addressing.regional_hostname_get_response import RegionalHostnameGetResponse
from ....types.addressing.regional_hostname_edit_response import RegionalHostnameEditResponse
from ....types.addressing.regional_hostname_list_response import RegionalHostnameListResponse
from ....types.addressing.regional_hostname_create_response import RegionalHostnameCreateResponse
from ....types.addressing.regional_hostname_delete_response import RegionalHostnameDeleteResponse

__all__ = ["RegionalHostnamesResource", "AsyncRegionalHostnamesResource"]


class RegionalHostnamesResource(SyncAPIResource):
    @cached_property
    def regions(self) -> RegionsResource:
        return RegionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> RegionalHostnamesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return RegionalHostnamesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RegionalHostnamesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return RegionalHostnamesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        zone_id: str,
        hostname: str,
        region_key: str,
        routing: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RegionalHostnameCreateResponse]:
        """Create a new Regional Hostname entry.

        Cloudflare will only use data centers that
        are physically located within the chosen region to decrypt and service HTTPS
        traffic. Learn more about
        [Regional Services](https://developers.cloudflare.com/data-localization/regional-services/get-started/).

        Args:
          zone_id: Identifier.

          hostname: DNS hostname to be regionalized, must be a subdomain of the zone. Wildcards are
              supported for one level, e.g `*.example.com`

          region_key: Identifying key for the region

          routing: Configure which routing method to use for the regional hostname

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._post(
            path_template("/zones/{zone_id}/addressing/regional_hostnames", zone_id=zone_id),
            body=maybe_transform(
                {
                    "hostname": hostname,
                    "region_key": region_key,
                    "routing": routing,
                },
                regional_hostname_create_params.RegionalHostnameCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RegionalHostnameCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RegionalHostnameCreateResponse]], ResultWrapper[RegionalHostnameCreateResponse]),
        )

    def list(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[RegionalHostnameListResponse]:
        """
        List all Regional Hostnames within a zone.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/addressing/regional_hostnames", zone_id=zone_id),
            page=SyncSinglePage[RegionalHostnameListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=RegionalHostnameListResponse,
        )

    def delete(
        self,
        hostname: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RegionalHostnameDeleteResponse:
        """
        Delete the region configuration for a specific Regional Hostname.

        Args:
          zone_id: Identifier.

          hostname: DNS hostname to be regionalized, must be a subdomain of the zone. Wildcards are
              supported for one level, e.g `*.example.com`

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not hostname:
            raise ValueError(f"Expected a non-empty value for `hostname` but received {hostname!r}")
        return self._delete(
            path_template(
                "/zones/{zone_id}/addressing/regional_hostnames/{hostname}", zone_id=zone_id, hostname=hostname
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RegionalHostnameDeleteResponse,
        )

    def edit(
        self,
        hostname: str,
        *,
        zone_id: str,
        region_key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RegionalHostnameEditResponse]:
        """Update the configuration for a specific Regional Hostname.

        Only the region_key
        of a hostname is mutable.

        Args:
          zone_id: Identifier.

          hostname: DNS hostname to be regionalized, must be a subdomain of the zone. Wildcards are
              supported for one level, e.g `*.example.com`

          region_key: Identifying key for the region

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not hostname:
            raise ValueError(f"Expected a non-empty value for `hostname` but received {hostname!r}")
        return self._patch(
            path_template(
                "/zones/{zone_id}/addressing/regional_hostnames/{hostname}", zone_id=zone_id, hostname=hostname
            ),
            body=maybe_transform({"region_key": region_key}, regional_hostname_edit_params.RegionalHostnameEditParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RegionalHostnameEditResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RegionalHostnameEditResponse]], ResultWrapper[RegionalHostnameEditResponse]),
        )

    def get(
        self,
        hostname: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RegionalHostnameGetResponse]:
        """
        Fetch the configuration for a specific Regional Hostname, within a zone.

        Args:
          zone_id: Identifier.

          hostname: DNS hostname to be regionalized, must be a subdomain of the zone. Wildcards are
              supported for one level, e.g `*.example.com`

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not hostname:
            raise ValueError(f"Expected a non-empty value for `hostname` but received {hostname!r}")
        return self._get(
            path_template(
                "/zones/{zone_id}/addressing/regional_hostnames/{hostname}", zone_id=zone_id, hostname=hostname
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RegionalHostnameGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RegionalHostnameGetResponse]], ResultWrapper[RegionalHostnameGetResponse]),
        )


class AsyncRegionalHostnamesResource(AsyncAPIResource):
    @cached_property
    def regions(self) -> AsyncRegionsResource:
        return AsyncRegionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncRegionalHostnamesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRegionalHostnamesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRegionalHostnamesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncRegionalHostnamesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        zone_id: str,
        hostname: str,
        region_key: str,
        routing: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RegionalHostnameCreateResponse]:
        """Create a new Regional Hostname entry.

        Cloudflare will only use data centers that
        are physically located within the chosen region to decrypt and service HTTPS
        traffic. Learn more about
        [Regional Services](https://developers.cloudflare.com/data-localization/regional-services/get-started/).

        Args:
          zone_id: Identifier.

          hostname: DNS hostname to be regionalized, must be a subdomain of the zone. Wildcards are
              supported for one level, e.g `*.example.com`

          region_key: Identifying key for the region

          routing: Configure which routing method to use for the regional hostname

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._post(
            path_template("/zones/{zone_id}/addressing/regional_hostnames", zone_id=zone_id),
            body=await async_maybe_transform(
                {
                    "hostname": hostname,
                    "region_key": region_key,
                    "routing": routing,
                },
                regional_hostname_create_params.RegionalHostnameCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RegionalHostnameCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RegionalHostnameCreateResponse]], ResultWrapper[RegionalHostnameCreateResponse]),
        )

    def list(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[RegionalHostnameListResponse, AsyncSinglePage[RegionalHostnameListResponse]]:
        """
        List all Regional Hostnames within a zone.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/addressing/regional_hostnames", zone_id=zone_id),
            page=AsyncSinglePage[RegionalHostnameListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=RegionalHostnameListResponse,
        )

    async def delete(
        self,
        hostname: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RegionalHostnameDeleteResponse:
        """
        Delete the region configuration for a specific Regional Hostname.

        Args:
          zone_id: Identifier.

          hostname: DNS hostname to be regionalized, must be a subdomain of the zone. Wildcards are
              supported for one level, e.g `*.example.com`

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not hostname:
            raise ValueError(f"Expected a non-empty value for `hostname` but received {hostname!r}")
        return await self._delete(
            path_template(
                "/zones/{zone_id}/addressing/regional_hostnames/{hostname}", zone_id=zone_id, hostname=hostname
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RegionalHostnameDeleteResponse,
        )

    async def edit(
        self,
        hostname: str,
        *,
        zone_id: str,
        region_key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RegionalHostnameEditResponse]:
        """Update the configuration for a specific Regional Hostname.

        Only the region_key
        of a hostname is mutable.

        Args:
          zone_id: Identifier.

          hostname: DNS hostname to be regionalized, must be a subdomain of the zone. Wildcards are
              supported for one level, e.g `*.example.com`

          region_key: Identifying key for the region

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not hostname:
            raise ValueError(f"Expected a non-empty value for `hostname` but received {hostname!r}")
        return await self._patch(
            path_template(
                "/zones/{zone_id}/addressing/regional_hostnames/{hostname}", zone_id=zone_id, hostname=hostname
            ),
            body=await async_maybe_transform(
                {"region_key": region_key}, regional_hostname_edit_params.RegionalHostnameEditParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RegionalHostnameEditResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RegionalHostnameEditResponse]], ResultWrapper[RegionalHostnameEditResponse]),
        )

    async def get(
        self,
        hostname: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RegionalHostnameGetResponse]:
        """
        Fetch the configuration for a specific Regional Hostname, within a zone.

        Args:
          zone_id: Identifier.

          hostname: DNS hostname to be regionalized, must be a subdomain of the zone. Wildcards are
              supported for one level, e.g `*.example.com`

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not hostname:
            raise ValueError(f"Expected a non-empty value for `hostname` but received {hostname!r}")
        return await self._get(
            path_template(
                "/zones/{zone_id}/addressing/regional_hostnames/{hostname}", zone_id=zone_id, hostname=hostname
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RegionalHostnameGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RegionalHostnameGetResponse]], ResultWrapper[RegionalHostnameGetResponse]),
        )


class RegionalHostnamesResourceWithRawResponse:
    def __init__(self, regional_hostnames: RegionalHostnamesResource) -> None:
        self._regional_hostnames = regional_hostnames

        self.create = to_raw_response_wrapper(
            regional_hostnames.create,
        )
        self.list = to_raw_response_wrapper(
            regional_hostnames.list,
        )
        self.delete = to_raw_response_wrapper(
            regional_hostnames.delete,
        )
        self.edit = to_raw_response_wrapper(
            regional_hostnames.edit,
        )
        self.get = to_raw_response_wrapper(
            regional_hostnames.get,
        )

    @cached_property
    def regions(self) -> RegionsResourceWithRawResponse:
        return RegionsResourceWithRawResponse(self._regional_hostnames.regions)


class AsyncRegionalHostnamesResourceWithRawResponse:
    def __init__(self, regional_hostnames: AsyncRegionalHostnamesResource) -> None:
        self._regional_hostnames = regional_hostnames

        self.create = async_to_raw_response_wrapper(
            regional_hostnames.create,
        )
        self.list = async_to_raw_response_wrapper(
            regional_hostnames.list,
        )
        self.delete = async_to_raw_response_wrapper(
            regional_hostnames.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            regional_hostnames.edit,
        )
        self.get = async_to_raw_response_wrapper(
            regional_hostnames.get,
        )

    @cached_property
    def regions(self) -> AsyncRegionsResourceWithRawResponse:
        return AsyncRegionsResourceWithRawResponse(self._regional_hostnames.regions)


class RegionalHostnamesResourceWithStreamingResponse:
    def __init__(self, regional_hostnames: RegionalHostnamesResource) -> None:
        self._regional_hostnames = regional_hostnames

        self.create = to_streamed_response_wrapper(
            regional_hostnames.create,
        )
        self.list = to_streamed_response_wrapper(
            regional_hostnames.list,
        )
        self.delete = to_streamed_response_wrapper(
            regional_hostnames.delete,
        )
        self.edit = to_streamed_response_wrapper(
            regional_hostnames.edit,
        )
        self.get = to_streamed_response_wrapper(
            regional_hostnames.get,
        )

    @cached_property
    def regions(self) -> RegionsResourceWithStreamingResponse:
        return RegionsResourceWithStreamingResponse(self._regional_hostnames.regions)


class AsyncRegionalHostnamesResourceWithStreamingResponse:
    def __init__(self, regional_hostnames: AsyncRegionalHostnamesResource) -> None:
        self._regional_hostnames = regional_hostnames

        self.create = async_to_streamed_response_wrapper(
            regional_hostnames.create,
        )
        self.list = async_to_streamed_response_wrapper(
            regional_hostnames.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            regional_hostnames.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            regional_hostnames.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            regional_hostnames.get,
        )

    @cached_property
    def regions(self) -> AsyncRegionsResourceWithStreamingResponse:
        return AsyncRegionsResourceWithStreamingResponse(self._regional_hostnames.regions)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import typing_extensions
from typing import List, Type, Optional, cast

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncV4PagePagination, AsyncV4PagePagination
from ...._base_client import AsyncPaginator, make_request_options
from ....types.intel.attack_surface_report import (
    issue_list_params,
    issue_type_params,
    issue_class_params,
    issue_dismiss_params,
    issue_severity_params,
)
from ....types.intel.attack_surface_report.issue_type import IssueType
from ....types.intel.attack_surface_report.issue_list_response import IssueListResponse
from ....types.intel.attack_surface_report.issue_type_response import IssueTypeResponse
from ....types.intel.attack_surface_report.issue_class_response import IssueClassResponse
from ....types.intel.attack_surface_report.severity_query_param import SeverityQueryParam
from ....types.intel.attack_surface_report.issue_dismiss_response import IssueDismissResponse
from ....types.intel.attack_surface_report.issue_severity_response import IssueSeverityResponse

__all__ = ["IssuesResource", "AsyncIssuesResource"]


class IssuesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> IssuesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return IssuesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> IssuesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return IssuesResourceWithStreamingResponse(self)

    @typing_extensions.deprecated("deprecated")
    def list(
        self,
        *,
        account_id: str,
        dismissed: bool | Omit = omit,
        issue_class: SequenceNotStr[str] | Omit = omit,
        issue_class_neq: SequenceNotStr[str] | Omit = omit,
        issue_type: List[IssueType] | Omit = omit,
        issue_type_neq: List[IssueType] | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        product: SequenceNotStr[str] | Omit = omit,
        product_neq: SequenceNotStr[str] | Omit = omit,
        severity: List[SeverityQueryParam] | Omit = omit,
        severity_neq: List[SeverityQueryParam] | Omit = omit,
        subject: SequenceNotStr[str] | Omit = omit,
        subject_neq: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePagination[Optional[IssueListResponse]]:
        """
        Lists all Security Center issues for the account, showing active security
        problems requiring attention.

        Args:
          account_id: Identifier.

          page: Specifies the current page within paginated list of results.

          per_page: Sets the number of results per page of results.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/intel/attack-surface-report/issues", account_id=account_id),
            page=SyncV4PagePagination[Optional[IssueListResponse]],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "dismissed": dismissed,
                        "issue_class": issue_class,
                        "issue_class_neq": issue_class_neq,
                        "issue_type": issue_type,
                        "issue_type_neq": issue_type_neq,
                        "page": page,
                        "per_page": per_page,
                        "product": product,
                        "product_neq": product_neq,
                        "severity": severity,
                        "severity_neq": severity_neq,
                        "subject": subject,
                        "subject_neq": subject_neq,
                    },
                    issue_list_params.IssueListParams,
                ),
            ),
            model=IssueListResponse,
        )

    @typing_extensions.deprecated("deprecated")
    def class_(
        self,
        *,
        account_id: str,
        dismissed: bool | Omit = omit,
        issue_class: SequenceNotStr[str] | Omit = omit,
        issue_class_neq: SequenceNotStr[str] | Omit = omit,
        issue_type: List[IssueType] | Omit = omit,
        issue_type_neq: List[IssueType] | Omit = omit,
        product: SequenceNotStr[str] | Omit = omit,
        product_neq: SequenceNotStr[str] | Omit = omit,
        severity: List[SeverityQueryParam] | Omit = omit,
        severity_neq: List[SeverityQueryParam] | Omit = omit,
        subject: SequenceNotStr[str] | Omit = omit,
        subject_neq: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[IssueClassResponse]:
        """
        Retrieves Security Center issue counts aggregated by classification class.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/intel/attack-surface-report/issues/class", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "dismissed": dismissed,
                        "issue_class": issue_class,
                        "issue_class_neq": issue_class_neq,
                        "issue_type": issue_type,
                        "issue_type_neq": issue_type_neq,
                        "product": product,
                        "product_neq": product_neq,
                        "severity": severity,
                        "severity_neq": severity_neq,
                        "subject": subject,
                        "subject_neq": subject_neq,
                    },
                    issue_class_params.IssueClassParams,
                ),
                post_parser=ResultWrapper[Optional[IssueClassResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[IssueClassResponse]], ResultWrapper[IssueClassResponse]),
        )

    @typing_extensions.deprecated("deprecated")
    def dismiss(
        self,
        issue_id: str,
        *,
        account_id: str,
        dismiss: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IssueDismissResponse:
        """Deprecated endpoint for archiving Security Center insights.

        Use the newer
        archive-security-center-insight endpoint instead.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not issue_id:
            raise ValueError(f"Expected a non-empty value for `issue_id` but received {issue_id!r}")
        return self._put(
            path_template(
                "/accounts/{account_id}/intel/attack-surface-report/{issue_id}/dismiss",
                account_id=account_id,
                issue_id=issue_id,
            ),
            body=maybe_transform({"dismiss": dismiss}, issue_dismiss_params.IssueDismissParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IssueDismissResponse,
        )

    @typing_extensions.deprecated("deprecated")
    def severity(
        self,
        *,
        account_id: str,
        dismissed: bool | Omit = omit,
        issue_class: SequenceNotStr[str] | Omit = omit,
        issue_class_neq: SequenceNotStr[str] | Omit = omit,
        issue_type: List[IssueType] | Omit = omit,
        issue_type_neq: List[IssueType] | Omit = omit,
        product: SequenceNotStr[str] | Omit = omit,
        product_neq: SequenceNotStr[str] | Omit = omit,
        severity: List[SeverityQueryParam] | Omit = omit,
        severity_neq: List[SeverityQueryParam] | Omit = omit,
        subject: SequenceNotStr[str] | Omit = omit,
        subject_neq: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[IssueSeverityResponse]:
        """
        Retrieves Security Center issue counts aggregated by severity level.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/intel/attack-surface-report/issues/severity", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "dismissed": dismissed,
                        "issue_class": issue_class,
                        "issue_class_neq": issue_class_neq,
                        "issue_type": issue_type,
                        "issue_type_neq": issue_type_neq,
                        "product": product,
                        "product_neq": product_neq,
                        "severity": severity,
                        "severity_neq": severity_neq,
                        "subject": subject,
                        "subject_neq": subject_neq,
                    },
                    issue_severity_params.IssueSeverityParams,
                ),
                post_parser=ResultWrapper[Optional[IssueSeverityResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[IssueSeverityResponse]], ResultWrapper[IssueSeverityResponse]),
        )

    @typing_extensions.deprecated("deprecated")
    def type(
        self,
        *,
        account_id: str,
        dismissed: bool | Omit = omit,
        issue_class: SequenceNotStr[str] | Omit = omit,
        issue_class_neq: SequenceNotStr[str] | Omit = omit,
        issue_type: List[IssueType] | Omit = omit,
        issue_type_neq: List[IssueType] | Omit = omit,
        product: SequenceNotStr[str] | Omit = omit,
        product_neq: SequenceNotStr[str] | Omit = omit,
        severity: List[SeverityQueryParam] | Omit = omit,
        severity_neq: List[SeverityQueryParam] | Omit = omit,
        subject: SequenceNotStr[str] | Omit = omit,
        subject_neq: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[IssueTypeResponse]:
        """
        Retrieves Security Center issue counts aggregated by issue type.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/intel/attack-surface-report/issues/type", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "dismissed": dismissed,
                        "issue_class": issue_class,
                        "issue_class_neq": issue_class_neq,
                        "issue_type": issue_type,
                        "issue_type_neq": issue_type_neq,
                        "product": product,
                        "product_neq": product_neq,
                        "severity": severity,
                        "severity_neq": severity_neq,
                        "subject": subject,
                        "subject_neq": subject_neq,
                    },
                    issue_type_params.IssueTypeParams,
                ),
                post_parser=ResultWrapper[Optional[IssueTypeResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[IssueTypeResponse]], ResultWrapper[IssueTypeResponse]),
        )


class AsyncIssuesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncIssuesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncIssuesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncIssuesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncIssuesResourceWithStreamingResponse(self)

    @typing_extensions.deprecated("deprecated")
    def list(
        self,
        *,
        account_id: str,
        dismissed: bool | Omit = omit,
        issue_class: SequenceNotStr[str] | Omit = omit,
        issue_class_neq: SequenceNotStr[str] | Omit = omit,
        issue_type: List[IssueType] | Omit = omit,
        issue_type_neq: List[IssueType] | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        product: SequenceNotStr[str] | Omit = omit,
        product_neq: SequenceNotStr[str] | Omit = omit,
        severity: List[SeverityQueryParam] | Omit = omit,
        severity_neq: List[SeverityQueryParam] | Omit = omit,
        subject: SequenceNotStr[str] | Omit = omit,
        subject_neq: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Optional[IssueListResponse], AsyncV4PagePagination[Optional[IssueListResponse]]]:
        """
        Lists all Security Center issues for the account, showing active security
        problems requiring attention.

        Args:
          account_id: Identifier.

          page: Specifies the current page within paginated list of results.

          per_page: Sets the number of results per page of results.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/intel/attack-surface-report/issues", account_id=account_id),
            page=AsyncV4PagePagination[Optional[IssueListResponse]],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "dismissed": dismissed,
                        "issue_class": issue_class,
                        "issue_class_neq": issue_class_neq,
                        "issue_type": issue_type,
                        "issue_type_neq": issue_type_neq,
                        "page": page,
                        "per_page": per_page,
                        "product": product,
                        "product_neq": product_neq,
                        "severity": severity,
                        "severity_neq": severity_neq,
                        "subject": subject,
                        "subject_neq": subject_neq,
                    },
                    issue_list_params.IssueListParams,
                ),
            ),
            model=IssueListResponse,
        )

    @typing_extensions.deprecated("deprecated")
    async def class_(
        self,
        *,
        account_id: str,
        dismissed: bool | Omit = omit,
        issue_class: SequenceNotStr[str] | Omit = omit,
        issue_class_neq: SequenceNotStr[str] | Omit = omit,
        issue_type: List[IssueType] | Omit = omit,
        issue_type_neq: List[IssueType] | Omit = omit,
        product: SequenceNotStr[str] | Omit = omit,
        product_neq: SequenceNotStr[str] | Omit = omit,
        severity: List[SeverityQueryParam] | Omit = omit,
        severity_neq: List[SeverityQueryParam] | Omit = omit,
        subject: SequenceNotStr[str] | Omit = omit,
        subject_neq: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[IssueClassResponse]:
        """
        Retrieves Security Center issue counts aggregated by classification class.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/intel/attack-surface-report/issues/class", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "dismissed": dismissed,
                        "issue_class": issue_class,
                        "issue_class_neq": issue_class_neq,
                        "issue_type": issue_type,
                        "issue_type_neq": issue_type_neq,
                        "product": product,
                        "product_neq": product_neq,
                        "severity": severity,
                        "severity_neq": severity_neq,
                        "subject": subject,
                        "subject_neq": subject_neq,
                    },
                    issue_class_params.IssueClassParams,
                ),
                post_parser=ResultWrapper[Optional[IssueClassResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[IssueClassResponse]], ResultWrapper[IssueClassResponse]),
        )

    @typing_extensions.deprecated("deprecated")
    async def dismiss(
        self,
        issue_id: str,
        *,
        account_id: str,
        dismiss: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IssueDismissResponse:
        """Deprecated endpoint for archiving Security Center insights.

        Use the newer
        archive-security-center-insight endpoint instead.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not issue_id:
            raise ValueError(f"Expected a non-empty value for `issue_id` but received {issue_id!r}")
        return await self._put(
            path_template(
                "/accounts/{account_id}/intel/attack-surface-report/{issue_id}/dismiss",
                account_id=account_id,
                issue_id=issue_id,
            ),
            body=await async_maybe_transform({"dismiss": dismiss}, issue_dismiss_params.IssueDismissParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IssueDismissResponse,
        )

    @typing_extensions.deprecated("deprecated")
    async def severity(
        self,
        *,
        account_id: str,
        dismissed: bool | Omit = omit,
        issue_class: SequenceNotStr[str] | Omit = omit,
        issue_class_neq: SequenceNotStr[str] | Omit = omit,
        issue_type: List[IssueType] | Omit = omit,
        issue_type_neq: List[IssueType] | Omit = omit,
        product: SequenceNotStr[str] | Omit = omit,
        product_neq: SequenceNotStr[str] | Omit = omit,
        severity: List[SeverityQueryParam] | Omit = omit,
        severity_neq: List[SeverityQueryParam] | Omit = omit,
        subject: SequenceNotStr[str] | Omit = omit,
        subject_neq: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[IssueSeverityResponse]:
        """
        Retrieves Security Center issue counts aggregated by severity level.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/intel/attack-surface-report/issues/severity", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "dismissed": dismissed,
                        "issue_class": issue_class,
                        "issue_class_neq": issue_class_neq,
                        "issue_type": issue_type,
                        "issue_type_neq": issue_type_neq,
                        "product": product,
                        "product_neq": product_neq,
                        "severity": severity,
                        "severity_neq": severity_neq,
                        "subject": subject,
                        "subject_neq": subject_neq,
                    },
                    issue_severity_params.IssueSeverityParams,
                ),
                post_parser=ResultWrapper[Optional[IssueSeverityResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[IssueSeverityResponse]], ResultWrapper[IssueSeverityResponse]),
        )

    @typing_extensions.deprecated("deprecated")
    async def type(
        self,
        *,
        account_id: str,
        dismissed: bool | Omit = omit,
        issue_class: SequenceNotStr[str] | Omit = omit,
        issue_class_neq: SequenceNotStr[str] | Omit = omit,
        issue_type: List[IssueType] | Omit = omit,
        issue_type_neq: List[IssueType] | Omit = omit,
        product: SequenceNotStr[str] | Omit = omit,
        product_neq: SequenceNotStr[str] | Omit = omit,
        severity: List[SeverityQueryParam] | Omit = omit,
        severity_neq: List[SeverityQueryParam] | Omit = omit,
        subject: SequenceNotStr[str] | Omit = omit,
        subject_neq: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[IssueTypeResponse]:
        """
        Retrieves Security Center issue counts aggregated by issue type.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/intel/attack-surface-report/issues/type", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "dismissed": dismissed,
                        "issue_class": issue_class,
                        "issue_class_neq": issue_class_neq,
                        "issue_type": issue_type,
                        "issue_type_neq": issue_type_neq,
                        "product": product,
                        "product_neq": product_neq,
                        "severity": severity,
                        "severity_neq": severity_neq,
                        "subject": subject,
                        "subject_neq": subject_neq,
                    },
                    issue_type_params.IssueTypeParams,
                ),
                post_parser=ResultWrapper[Optional[IssueTypeResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[IssueTypeResponse]], ResultWrapper[IssueTypeResponse]),
        )


class IssuesResourceWithRawResponse:
    def __init__(self, issues: IssuesResource) -> None:
        self._issues = issues

        self.list = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                issues.list,  # pyright: ignore[reportDeprecated],
            )
        )
        self.class_ = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                issues.class_,  # pyright: ignore[reportDeprecated],
            )
        )
        self.dismiss = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                issues.dismiss,  # pyright: ignore[reportDeprecated],
            )
        )
        self.severity = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                issues.severity,  # pyright: ignore[reportDeprecated],
            )
        )
        self.type = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                issues.type,  # pyright: ignore[reportDeprecated],
            )
        )


class AsyncIssuesResourceWithRawResponse:
    def __init__(self, issues: AsyncIssuesResource) -> None:
        self._issues = issues

        self.list = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                issues.list,  # pyright: ignore[reportDeprecated],
            )
        )
        self.class_ = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                issues.class_,  # pyright: ignore[reportDeprecated],
            )
        )
        self.dismiss = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                issues.dismiss,  # pyright: ignore[reportDeprecated],
            )
        )
        self.severity = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                issues.severity,  # pyright: ignore[reportDeprecated],
            )
        )
        self.type = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                issues.type,  # pyright: ignore[reportDeprecated],
            )
        )


class IssuesResourceWithStreamingResponse:
    def __init__(self, issues: IssuesResource) -> None:
        self._issues = issues

        self.list = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                issues.list,  # pyright: ignore[reportDeprecated],
            )
        )
        self.class_ = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                issues.class_,  # pyright: ignore[reportDeprecated],
            )
        )
        self.dismiss = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                issues.dismiss,  # pyright: ignore[reportDeprecated],
            )
        )
        self.severity = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                issues.severity,  # pyright: ignore[reportDeprecated],
            )
        )
        self.type = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                issues.type,  # pyright: ignore[reportDeprecated],
            )
        )


class AsyncIssuesResourceWithStreamingResponse:
    def __init__(self, issues: AsyncIssuesResource) -> None:
        self._issues = issues

        self.list = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                issues.list,  # pyright: ignore[reportDeprecated],
            )
        )
        self.class_ = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                issues.class_,  # pyright: ignore[reportDeprecated],
            )
        )
        self.dismiss = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                issues.dismiss,  # pyright: ignore[reportDeprecated],
            )
        )
        self.severity = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                issues.severity,  # pyright: ignore[reportDeprecated],
            )
        )
        self.type = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                issues.type,  # pyright: ignore[reportDeprecated],
            )
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["SessionGetSessionParticipantDetailsParams"]


class SessionGetSessionParticipantDetailsParams(TypedDict, total=False):
    account_id: Required[str]
    """The account identifier tag."""

    app_id: Required[str]
    """The app identifier tag."""

    session_id: Required[str]

    filters: Literal["device_info", "ip_information", "precall_network_information", "events", "quality_stats"]
    """Comma separated list of filters to apply.

    Note that there must be no spaces between the filters.
    """

    include_peer_events: bool
    """if true, response includes all the peer events of participant."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .currents import (
    CurrentsResource,
    AsyncCurrentsResource,
    CurrentsResourceWithRawResponse,
    AsyncCurrentsResourceWithRawResponse,
    CurrentsResourceWithStreamingResponse,
    AsyncCurrentsResourceWithStreamingResponse,
)
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["AggregatesResource", "AsyncAggregatesResource"]


class AggregatesResource(SyncAPIResource):
    @cached_property
    def currents(self) -> CurrentsResource:
        return CurrentsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AggregatesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AggregatesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AggregatesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AggregatesResourceWithStreamingResponse(self)


class AsyncAggregatesResource(AsyncAPIResource):
    @cached_property
    def currents(self) -> AsyncCurrentsResource:
        return AsyncCurrentsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncAggregatesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAggregatesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAggregatesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncAggregatesResourceWithStreamingResponse(self)


class AggregatesResourceWithRawResponse:
    def __init__(self, aggregates: AggregatesResource) -> None:
        self._aggregates = aggregates

    @cached_property
    def currents(self) -> CurrentsResourceWithRawResponse:
        return CurrentsResourceWithRawResponse(self._aggregates.currents)


class AsyncAggregatesResourceWithRawResponse:
    def __init__(self, aggregates: AsyncAggregatesResource) -> None:
        self._aggregates = aggregates

    @cached_property
    def currents(self) -> AsyncCurrentsResourceWithRawResponse:
        return AsyncCurrentsResourceWithRawResponse(self._aggregates.currents)


class AggregatesResourceWithStreamingResponse:
    def __init__(self, aggregates: AggregatesResource) -> None:
        self._aggregates = aggregates

    @cached_property
    def currents(self) -> CurrentsResourceWithStreamingResponse:
        return CurrentsResourceWithStreamingResponse(self._aggregates.currents)


class AsyncAggregatesResourceWithStreamingResponse:
    def __init__(self, aggregates: AsyncAggregatesResource) -> None:
        self._aggregates = aggregates

    @cached_property
    def currents(self) -> AsyncCurrentsResourceWithStreamingResponse:
        return AsyncCurrentsResourceWithStreamingResponse(self._aggregates.currents)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union
from typing_extensions import TypeAlias

__all__ = ["RevokeCreateResponse"]

RevokeCreateResponse: TypeAlias = Union[str, object, None]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .tokens import (
    TokensResource,
    AsyncTokensResource,
    TokensResourceWithRawResponse,
    AsyncTokensResourceWithRawResponse,
    TokensResourceWithStreamingResponse,
    AsyncTokensResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .instances.instances import (
    InstancesResource,
    AsyncInstancesResource,
    InstancesResourceWithRawResponse,
    AsyncInstancesResourceWithRawResponse,
    InstancesResourceWithStreamingResponse,
    AsyncInstancesResourceWithStreamingResponse,
)
from .namespaces.namespaces import (
    NamespacesResource,
    AsyncNamespacesResource,
    NamespacesResourceWithRawResponse,
    AsyncNamespacesResourceWithRawResponse,
    NamespacesResourceWithStreamingResponse,
    AsyncNamespacesResourceWithStreamingResponse,
)

__all__ = ["AISearchResource", "AsyncAISearchResource"]


class AISearchResource(SyncAPIResource):
    @cached_property
    def namespaces(self) -> NamespacesResource:
        return NamespacesResource(self._client)

    @cached_property
    def instances(self) -> InstancesResource:
        return InstancesResource(self._client)

    @cached_property
    def tokens(self) -> TokensResource:
        return TokensResource(self._client)

    @cached_property
    def with_raw_response(self) -> AISearchResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AISearchResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AISearchResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AISearchResourceWithStreamingResponse(self)


class AsyncAISearchResource(AsyncAPIResource):
    @cached_property
    def namespaces(self) -> AsyncNamespacesResource:
        return AsyncNamespacesResource(self._client)

    @cached_property
    def instances(self) -> AsyncInstancesResource:
        return AsyncInstancesResource(self._client)

    @cached_property
    def tokens(self) -> AsyncTokensResource:
        return AsyncTokensResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncAISearchResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAISearchResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAISearchResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncAISearchResourceWithStreamingResponse(self)


class AISearchResourceWithRawResponse:
    def __init__(self, aisearch: AISearchResource) -> None:
        self._aisearch = aisearch

    @cached_property
    def namespaces(self) -> NamespacesResourceWithRawResponse:
        return NamespacesResourceWithRawResponse(self._aisearch.namespaces)

    @cached_property
    def instances(self) -> InstancesResourceWithRawResponse:
        return InstancesResourceWithRawResponse(self._aisearch.instances)

    @cached_property
    def tokens(self) -> TokensResourceWithRawResponse:
        return TokensResourceWithRawResponse(self._aisearch.tokens)


class AsyncAISearchResourceWithRawResponse:
    def __init__(self, aisearch: AsyncAISearchResource) -> None:
        self._aisearch = aisearch

    @cached_property
    def namespaces(self) -> AsyncNamespacesResourceWithRawResponse:
        return AsyncNamespacesResourceWithRawResponse(self._aisearch.namespaces)

    @cached_property
    def instances(self) -> AsyncInstancesResourceWithRawResponse:
        return AsyncInstancesResourceWithRawResponse(self._aisearch.instances)

    @cached_property
    def tokens(self) -> AsyncTokensResourceWithRawResponse:
        return AsyncTokensResourceWithRawResponse(self._aisearch.tokens)


class AISearchResourceWithStreamingResponse:
    def __init__(self, aisearch: AISearchResource) -> None:
        self._aisearch = aisearch

    @cached_property
    def namespaces(self) -> NamespacesResourceWithStreamingResponse:
        return NamespacesResourceWithStreamingResponse(self._aisearch.namespaces)

    @cached_property
    def instances(self) -> InstancesResourceWithStreamingResponse:
        return InstancesResourceWithStreamingResponse(self._aisearch.instances)

    @cached_property
    def tokens(self) -> TokensResourceWithStreamingResponse:
        return TokensResourceWithStreamingResponse(self._aisearch.tokens)


class AsyncAISearchResourceWithStreamingResponse:
    def __init__(self, aisearch: AsyncAISearchResource) -> None:
        self._aisearch = aisearch

    @cached_property
    def namespaces(self) -> AsyncNamespacesResourceWithStreamingResponse:
        return AsyncNamespacesResourceWithStreamingResponse(self._aisearch.namespaces)

    @cached_property
    def instances(self) -> AsyncInstancesResourceWithStreamingResponse:
        return AsyncInstancesResourceWithStreamingResponse(self._aisearch.instances)

    @cached_property
    def tokens(self) -> AsyncTokensResourceWithStreamingResponse:
        return AsyncTokensResourceWithStreamingResponse(self._aisearch.tokens)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .asn import (
    ASNResource,
    AsyncASNResource,
    ASNResourceWithRawResponse,
    AsyncASNResourceWithRawResponse,
    ASNResourceWithStreamingResponse,
    AsyncASNResourceWithStreamingResponse,
)
from .configs import (
    ConfigsResource,
    AsyncConfigsResource,
    ConfigsResourceWithRawResponse,
    AsyncConfigsResourceWithRawResponse,
    ConfigsResourceWithStreamingResponse,
    AsyncConfigsResourceWithStreamingResponse,
)
from .botnet_feed import (
    BotnetFeedResource,
    AsyncBotnetFeedResource,
    BotnetFeedResourceWithRawResponse,
    AsyncBotnetFeedResourceWithRawResponse,
    BotnetFeedResourceWithStreamingResponse,
    AsyncBotnetFeedResourceWithStreamingResponse,
)

__all__ = [
    "ASNResource",
    "AsyncASNResource",
    "ASNResourceWithRawResponse",
    "AsyncASNResourceWithRawResponse",
    "ASNResourceWithStreamingResponse",
    "AsyncASNResourceWithStreamingResponse",
    "ConfigsResource",
    "AsyncConfigsResource",
    "ConfigsResourceWithRawResponse",
    "AsyncConfigsResourceWithRawResponse",
    "ConfigsResourceWithStreamingResponse",
    "AsyncConfigsResourceWithStreamingResponse",
    "BotnetFeedResource",
    "AsyncBotnetFeedResource",
    "BotnetFeedResourceWithRawResponse",
    "AsyncBotnetFeedResourceWithRawResponse",
    "BotnetFeedResourceWithStreamingResponse",
    "AsyncBotnetFeedResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Iterable, cast

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ..._base_client import make_request_options
from ...types.ai_gateway import (
    dynamic_routing_list_params,
    dynamic_routing_create_params,
    dynamic_routing_update_params,
    dynamic_routing_create_version_params,
    dynamic_routing_create_deployment_params,
)
from ...types.ai_gateway.dynamic_routing_get_response import DynamicRoutingGetResponse
from ...types.ai_gateway.dynamic_routing_list_response import DynamicRoutingListResponse
from ...types.ai_gateway.dynamic_routing_create_response import DynamicRoutingCreateResponse
from ...types.ai_gateway.dynamic_routing_delete_response import DynamicRoutingDeleteResponse
from ...types.ai_gateway.dynamic_routing_update_response import DynamicRoutingUpdateResponse
from ...types.ai_gateway.dynamic_routing_get_version_response import DynamicRoutingGetVersionResponse
from ...types.ai_gateway.dynamic_routing_list_versions_response import DynamicRoutingListVersionsResponse
from ...types.ai_gateway.dynamic_routing_create_version_response import DynamicRoutingCreateVersionResponse
from ...types.ai_gateway.dynamic_routing_list_deployments_response import DynamicRoutingListDeploymentsResponse
from ...types.ai_gateway.dynamic_routing_create_deployment_response import DynamicRoutingCreateDeploymentResponse

__all__ = ["DynamicRoutingResource", "AsyncDynamicRoutingResource"]


class DynamicRoutingResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DynamicRoutingResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return DynamicRoutingResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DynamicRoutingResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return DynamicRoutingResourceWithStreamingResponse(self)

    def create(
        self,
        gateway_id: str,
        *,
        account_id: str,
        elements: Iterable[dynamic_routing_create_params.Element],
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DynamicRoutingCreateResponse:
        """
        Create a new AI Gateway Dynamic Route.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/routes",
                account_id=account_id,
                gateway_id=gateway_id,
            ),
            body=maybe_transform(
                {
                    "elements": elements,
                    "name": name,
                },
                dynamic_routing_create_params.DynamicRoutingCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[DynamicRoutingCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[DynamicRoutingCreateResponse], ResultWrapper[DynamicRoutingCreateResponse]),
        )

    def update(
        self,
        id: str,
        *,
        account_id: str,
        gateway_id: str,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DynamicRoutingUpdateResponse:
        """
        Update an AI Gateway Dynamic Route.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/routes/{id}",
                account_id=account_id,
                gateway_id=gateway_id,
                id=id,
            ),
            body=maybe_transform({"name": name}, dynamic_routing_update_params.DynamicRoutingUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DynamicRoutingUpdateResponse,
        )

    def list(
        self,
        gateway_id: str,
        *,
        account_id: str,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DynamicRoutingListResponse:
        """
        List all AI Gateway Dynamic Routes.

        Args:
          page: Page number

          per_page: Number of routes per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/routes",
                account_id=account_id,
                gateway_id=gateway_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                    },
                    dynamic_routing_list_params.DynamicRoutingListParams,
                ),
            ),
            cast_to=DynamicRoutingListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        account_id: str,
        gateway_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DynamicRoutingDeleteResponse:
        """
        Delete an AI Gateway Dynamic Route.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/routes/{id}",
                account_id=account_id,
                gateway_id=gateway_id,
                id=id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[DynamicRoutingDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[DynamicRoutingDeleteResponse], ResultWrapper[DynamicRoutingDeleteResponse]),
        )

    def create_deployment(
        self,
        id: str,
        *,
        account_id: str,
        gateway_id: str,
        version_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DynamicRoutingCreateDeploymentResponse:
        """
        Create a new AI Gateway Dynamic Route Deployment.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/routes/{id}/deployments",
                account_id=account_id,
                gateway_id=gateway_id,
                id=id,
            ),
            body=maybe_transform(
                {"version_id": version_id},
                dynamic_routing_create_deployment_params.DynamicRoutingCreateDeploymentParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[DynamicRoutingCreateDeploymentResponse]._unwrapper,
            ),
            cast_to=cast(
                Type[DynamicRoutingCreateDeploymentResponse], ResultWrapper[DynamicRoutingCreateDeploymentResponse]
            ),
        )

    def create_version(
        self,
        id: str,
        *,
        account_id: str,
        gateway_id: str,
        elements: Iterable[dynamic_routing_create_version_params.Element],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DynamicRoutingCreateVersionResponse:
        """
        Create a new AI Gateway Dynamic Route Version.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/routes/{id}/versions",
                account_id=account_id,
                gateway_id=gateway_id,
                id=id,
            ),
            body=maybe_transform(
                {"elements": elements}, dynamic_routing_create_version_params.DynamicRoutingCreateVersionParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[DynamicRoutingCreateVersionResponse]._unwrapper,
            ),
            cast_to=cast(Type[DynamicRoutingCreateVersionResponse], ResultWrapper[DynamicRoutingCreateVersionResponse]),
        )

    def get(
        self,
        id: str,
        *,
        account_id: str,
        gateway_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DynamicRoutingGetResponse:
        """
        Get an AI Gateway Dynamic Route.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/routes/{id}",
                account_id=account_id,
                gateway_id=gateway_id,
                id=id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[DynamicRoutingGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[DynamicRoutingGetResponse], ResultWrapper[DynamicRoutingGetResponse]),
        )

    def get_version(
        self,
        version_id: str,
        *,
        account_id: str,
        gateway_id: str,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DynamicRoutingGetVersionResponse:
        """
        Get an AI Gateway Dynamic Route Version.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not version_id:
            raise ValueError(f"Expected a non-empty value for `version_id` but received {version_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/routes/{id}/versions/{version_id}",
                account_id=account_id,
                gateway_id=gateway_id,
                id=id,
                version_id=version_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[DynamicRoutingGetVersionResponse]._unwrapper,
            ),
            cast_to=cast(Type[DynamicRoutingGetVersionResponse], ResultWrapper[DynamicRoutingGetVersionResponse]),
        )

    def list_deployments(
        self,
        id: str,
        *,
        account_id: str,
        gateway_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DynamicRoutingListDeploymentsResponse:
        """
        List all AI Gateway Dynamic Route Deployments.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/routes/{id}/deployments",
                account_id=account_id,
                gateway_id=gateway_id,
                id=id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DynamicRoutingListDeploymentsResponse,
        )

    def list_versions(
        self,
        id: str,
        *,
        account_id: str,
        gateway_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DynamicRoutingListVersionsResponse:
        """
        List all AI Gateway Dynamic Route Versions.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/routes/{id}/versions",
                account_id=account_id,
                gateway_id=gateway_id,
                id=id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DynamicRoutingListVersionsResponse,
        )


class AsyncDynamicRoutingResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDynamicRoutingResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDynamicRoutingResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDynamicRoutingResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncDynamicRoutingResourceWithStreamingResponse(self)

    async def create(
        self,
        gateway_id: str,
        *,
        account_id: str,
        elements: Iterable[dynamic_routing_create_params.Element],
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DynamicRoutingCreateResponse:
        """
        Create a new AI Gateway Dynamic Route.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/routes",
                account_id=account_id,
                gateway_id=gateway_id,
            ),
            body=await async_maybe_transform(
                {
                    "elements": elements,
                    "name": name,
                },
                dynamic_routing_create_params.DynamicRoutingCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[DynamicRoutingCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[DynamicRoutingCreateResponse], ResultWrapper[DynamicRoutingCreateResponse]),
        )

    async def update(
        self,
        id: str,
        *,
        account_id: str,
        gateway_id: str,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DynamicRoutingUpdateResponse:
        """
        Update an AI Gateway Dynamic Route.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/routes/{id}",
                account_id=account_id,
                gateway_id=gateway_id,
                id=id,
            ),
            body=await async_maybe_transform({"name": name}, dynamic_routing_update_params.DynamicRoutingUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DynamicRoutingUpdateResponse,
        )

    async def list(
        self,
        gateway_id: str,
        *,
        account_id: str,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DynamicRoutingListResponse:
        """
        List all AI Gateway Dynamic Routes.

        Args:
          page: Page number

          per_page: Number of routes per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/routes",
                account_id=account_id,
                gateway_id=gateway_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                    },
                    dynamic_routing_list_params.DynamicRoutingListParams,
                ),
            ),
            cast_to=DynamicRoutingListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        account_id: str,
        gateway_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DynamicRoutingDeleteResponse:
        """
        Delete an AI Gateway Dynamic Route.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/routes/{id}",
                account_id=account_id,
                gateway_id=gateway_id,
                id=id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[DynamicRoutingDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[DynamicRoutingDeleteResponse], ResultWrapper[DynamicRoutingDeleteResponse]),
        )

    async def create_deployment(
        self,
        id: str,
        *,
        account_id: str,
        gateway_id: str,
        version_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DynamicRoutingCreateDeploymentResponse:
        """
        Create a new AI Gateway Dynamic Route Deployment.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/routes/{id}/deployments",
                account_id=account_id,
                gateway_id=gateway_id,
                id=id,
            ),
            body=await async_maybe_transform(
                {"version_id": version_id},
                dynamic_routing_create_deployment_params.DynamicRoutingCreateDeploymentParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[DynamicRoutingCreateDeploymentResponse]._unwrapper,
            ),
            cast_to=cast(
                Type[DynamicRoutingCreateDeploymentResponse], ResultWrapper[DynamicRoutingCreateDeploymentResponse]
            ),
        )

    async def create_version(
        self,
        id: str,
        *,
        account_id: str,
        gateway_id: str,
        elements: Iterable[dynamic_routing_create_version_params.Element],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DynamicRoutingCreateVersionResponse:
        """
        Create a new AI Gateway Dynamic Route Version.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/routes/{id}/versions",
                account_id=account_id,
                gateway_id=gateway_id,
                id=id,
            ),
            body=await async_maybe_transform(
                {"elements": elements}, dynamic_routing_create_version_params.DynamicRoutingCreateVersionParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[DynamicRoutingCreateVersionResponse]._unwrapper,
            ),
            cast_to=cast(Type[DynamicRoutingCreateVersionResponse], ResultWrapper[DynamicRoutingCreateVersionResponse]),
        )

    async def get(
        self,
        id: str,
        *,
        account_id: str,
        gateway_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DynamicRoutingGetResponse:
        """
        Get an AI Gateway Dynamic Route.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/routes/{id}",
                account_id=account_id,
                gateway_id=gateway_id,
                id=id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[DynamicRoutingGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[DynamicRoutingGetResponse], ResultWrapper[DynamicRoutingGetResponse]),
        )

    async def get_version(
        self,
        version_id: str,
        *,
        account_id: str,
        gateway_id: str,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DynamicRoutingGetVersionResponse:
        """
        Get an AI Gateway Dynamic Route Version.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not version_id:
            raise ValueError(f"Expected a non-empty value for `version_id` but received {version_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/routes/{id}/versions/{version_id}",
                account_id=account_id,
                gateway_id=gateway_id,
                id=id,
                version_id=version_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[DynamicRoutingGetVersionResponse]._unwrapper,
            ),
            cast_to=cast(Type[DynamicRoutingGetVersionResponse], ResultWrapper[DynamicRoutingGetVersionResponse]),
        )

    async def list_deployments(
        self,
        id: str,
        *,
        account_id: str,
        gateway_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DynamicRoutingListDeploymentsResponse:
        """
        List all AI Gateway Dynamic Route Deployments.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/routes/{id}/deployments",
                account_id=account_id,
                gateway_id=gateway_id,
                id=id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DynamicRoutingListDeploymentsResponse,
        )

    async def list_versions(
        self,
        id: str,
        *,
        account_id: str,
        gateway_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DynamicRoutingListVersionsResponse:
        """
        List all AI Gateway Dynamic Route Versions.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/routes/{id}/versions",
                account_id=account_id,
                gateway_id=gateway_id,
                id=id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DynamicRoutingListVersionsResponse,
        )


class DynamicRoutingResourceWithRawResponse:
    def __init__(self, dynamic_routing: DynamicRoutingResource) -> None:
        self._dynamic_routing = dynamic_routing

        self.create = to_raw_response_wrapper(
            dynamic_routing.create,
        )
        self.update = to_raw_response_wrapper(
            dynamic_routing.update,
        )
        self.list = to_raw_response_wrapper(
            dynamic_routing.list,
        )
        self.delete = to_raw_response_wrapper(
            dynamic_routing.delete,
        )
        self.create_deployment = to_raw_response_wrapper(
            dynamic_routing.create_deployment,
        )
        self.create_version = to_raw_response_wrapper(
            dynamic_routing.create_version,
        )
        self.get = to_raw_response_wrapper(
            dynamic_routing.get,
        )
        self.get_version = to_raw_response_wrapper(
            dynamic_routing.get_version,
        )
        self.list_deployments = to_raw_response_wrapper(
            dynamic_routing.list_deployments,
        )
        self.list_versions = to_raw_response_wrapper(
            dynamic_routing.list_versions,
        )


class AsyncDynamicRoutingResourceWithRawResponse:
    def __init__(self, dynamic_routing: AsyncDynamicRoutingResource) -> None:
        self._dynamic_routing = dynamic_routing

        self.create = async_to_raw_response_wrapper(
            dynamic_routing.create,
        )
        self.update = async_to_raw_response_wrapper(
            dynamic_routing.update,
        )
        self.list = async_to_raw_response_wrapper(
            dynamic_routing.list,
        )
        self.delete = async_to_raw_response_wrapper(
            dynamic_routing.delete,
        )
        self.create_deployment = async_to_raw_response_wrapper(
            dynamic_routing.create_deployment,
        )
        self.create_version = async_to_raw_response_wrapper(
            dynamic_routing.create_version,
        )
        self.get = async_to_raw_response_wrapper(
            dynamic_routing.get,
        )
        self.get_version = async_to_raw_response_wrapper(
            dynamic_routing.get_version,
        )
        self.list_deployments = async_to_raw_response_wrapper(
            dynamic_routing.list_deployments,
        )
        self.list_versions = async_to_raw_response_wrapper(
            dynamic_routing.list_versions,
        )


class DynamicRoutingResourceWithStreamingResponse:
    def __init__(self, dynamic_routing: DynamicRoutingResource) -> None:
        self._dynamic_routing = dynamic_routing

        self.create = to_streamed_response_wrapper(
            dynamic_routing.create,
        )
        self.update = to_streamed_response_wrapper(
            dynamic_routing.update,
        )
        self.list = to_streamed_response_wrapper(
            dynamic_routing.list,
        )
        self.delete = to_streamed_response_wrapper(
            dynamic_routing.delete,
        )
        self.create_deployment = to_streamed_response_wrapper(
            dynamic_routing.create_deployment,
        )
        self.create_version = to_streamed_response_wrapper(
            dynamic_routing.create_version,
        )
        self.get = to_streamed_response_wrapper(
            dynamic_routing.get,
        )
        self.get_version = to_streamed_response_wrapper(
            dynamic_routing.get_version,
        )
        self.list_deployments = to_streamed_response_wrapper(
            dynamic_routing.list_deployments,
        )
        self.list_versions = to_streamed_response_wrapper(
            dynamic_routing.list_versions,
        )


class AsyncDynamicRoutingResourceWithStreamingResponse:
    def __init__(self, dynamic_routing: AsyncDynamicRoutingResource) -> None:
        self._dynamic_routing = dynamic_routing

        self.create = async_to_streamed_response_wrapper(
            dynamic_routing.create,
        )
        self.update = async_to_streamed_response_wrapper(
            dynamic_routing.update,
        )
        self.list = async_to_streamed_response_wrapper(
            dynamic_routing.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            dynamic_routing.delete,
        )
        self.create_deployment = async_to_streamed_response_wrapper(
            dynamic_routing.create_deployment,
        )
        self.create_version = async_to_streamed_response_wrapper(
            dynamic_routing.create_version,
        )
        self.get = async_to_streamed_response_wrapper(
            dynamic_routing.get,
        )
        self.get_version = async_to_streamed_response_wrapper(
            dynamic_routing.get_version,
        )
        self.list_deployments = async_to_streamed_response_wrapper(
            dynamic_routing.list_deployments,
        )
        self.list_versions = async_to_streamed_response_wrapper(
            dynamic_routing.list_versions,
        )
